/*
 * Generate crypto-quality random bytes.  Or, if not, then at least generate
 * random-ish bytes.
 *
 * Copyright (C) 2001  Robey Pointer <robey@lag.net>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* only build this file if openssl is installed */
#ifdef HAVE_OPENSSL
#include <openssl/rand.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <glib.h>
#include "rand.h"

static gboolean
try_dev_urandom (unsigned char *buffer, int len)
{
	static gboolean urandom_missing = FALSE;
	static int fd = -1;

	if (urandom_missing) {
		return FALSE;
	}

	if (fd < 0) {
		fd = open("/dev/urandom", O_RDONLY);
		if (fd < 0) {
			urandom_missing = TRUE;
			return FALSE;
		}
	}

	if (read(fd, buffer, len) < len) {
		close(fd);
		fd = -1;
		return FALSE;
	}

        return TRUE;
}

void
say2_make_random (unsigned char *buffer, int len)
{
        struct timeval tv;
        pid_t pid;
	int i;

#ifdef HAVE_OPENSSL
        if (RAND_bytes(buffer, len) == 1) {
                return;
        }
#endif
        if (try_dev_urandom(buffer, len)) {
                return;
        }

        /* hmm, try using srand/rand */
        gettimeofday(&tv, NULL);
        pid = getpid();
        srand(tv.tv_usec % (int)pid);

        for (i = 0; i < len; i++) {
                buffer[i] = (unsigned char) (256.0*rand()/(RAND_MAX+1.0));
        }
}

